/********************************************************************************************
* TITLE: 			Fixed effect logistic regression for diagnostic accuracy meta-analysis  *
* AUTHOR: 		    Yemisi Takwoingi														*
* DATE CREATED: 	27/04/2014 																*
* DATE MODIFIED: 	26/09/2021																*
********************************************************************************************/

/* Set your working directory to the appropriate drive where you saved the file "Xpert Ultra_sputum_composite reference standard.csv". Replace "U:\Handbook 2020" with your path.*/
cd "U:\Handbook 2020"

*************** 1. IMPORT DATA *********

*** Read in the data from the .csv file. Note you should change the file path to where you saved the .csv file. ***
insheet using "Xpert Ultra_sputum_composite reference standard.csv", comma clear

*** Produce a summary of the dataset to check data import was ok. ***
describe 

************* 2. SET UP THE DATA BEFORE THE META-ANALYSIS  ********************

/* Generate 5 new variables of type long. We need these before we can reshape the data.
• n1 is number diseased
• n0 is number without disease
• true1 is number of true positives
• true0 is the number of true negatives
• study is the unique identifier for each study. _n will generate a sequence of numbers.*/
gen long n1=tp+fn
gen long n0=fp+tn
gen long true1=tp
gen long true0=tn
gen long recordid= _n 

*** Reshape the data from wide to long format. ***
reshape long n true, i(recordid) j(sens)

*** Generate a new binary variable spec of type byte that takes the value 0 when sens=1 and vice versa. ***	
gen byte spec=1-sens

*** Sort data to ensure studies are clustered together first by study.	***
sort studyid sens

******************* 3. PERFORM META-ANALYSIS **********************************

* Fixed effect logistic regression
blogit true n sens spec, nocons


********************** 4. DISPLAY SUMMARY ESTIMATES *******************************************************

*** Drop the program in case it is already in Stata’s memory. *** 
capture program drop renamematrix

/* Write a little program renaming elements of the coefficient and variance matrices. 	*/
program define renamematrix, eclass
matrix mb = e(b)
matrix mv=e(V)
matrix colnames mb = logitse:_cons logitsp:_cons 
matrix colnames mv = logitse:_cons logitsp:_cons 
matrix rownames mv = logitse:_cons logitsp:_cons 
ereturn post mb mv
end

*** Run the program. ***
renamematrix

*** Display the summary estimates for sensitivity and specificity. ***
_diparm logitse, label(Sensitivity) invlogit 
_diparm logitsp, label(Specificity) invlogit 
_diparm logitse logitsp, label(LR+ ) ci(log) function(invlogit(@1)/(1-invlogit(@2))) derivative(exp(@2-1)*invlogit(@1)^2/invlogit(@2) exp(@2)*invlogit(@1))
_diparm logitse logitsp, label(LR- ) ci(log) function((1-invlogit(@1))/invlogit(@2)) derivative(exp(-@1)*invlogit(@1)^2/invlogit(@2)  exp(-@1-@2)*invlogit(@1))  
